@@ -69,6 +69,7 @@ DWORD wined3d_usage_from_d3d11(enum D3D11_USAGE usage) DECLSPEC_HIDDEN;
 struct wined3d_resource *wined3d_resource_from_d3d11_resource(ID3D11Resource *resource) DECLSPEC_HIDDEN;
 struct wined3d_resource *wined3d_resource_from_d3d10_resource(ID3D10Resource *resource) DECLSPEC_HIDDEN;
 DWORD wined3d_map_flags_from_d3d11_map_type(D3D11_MAP map_type) DECLSPEC_HIDDEN;
+DWORD wined3d_map_flags_from_d3d10_map_type(D3D10_MAP map_type) DECLSPEC_HIDDEN;
 DWORD wined3d_clear_flags_from_d3d11_clear_flags(UINT clear_flags) DECLSPEC_HIDDEN;
 unsigned int wined3d_access_from_d3d11(D3D11_USAGE usage, UINT cpu_access) DECLSPEC_HIDDEN;
 
@@ -92,14 +93,33 @@ HRESULT d3d_set_private_data(struct wined3d_private_store *store,
 HRESULT d3d_set_private_data_interface(struct wined3d_private_store *store,
         REFGUID guid, const IUnknown *object) DECLSPEC_HIDDEN;
 
-static inline unsigned int wined3d_bind_flags_from_d3d11(UINT bind_flags)
+static inline unsigned int wined3d_bind_flags_from_d3d11(UINT bind_flags, UINT misc_flags)
 {
-    return bind_flags;
+    unsigned int wined3d_flags = bind_flags & (D3D11_BIND_VERTEX_BUFFER
+            | D3D11_BIND_INDEX_BUFFER
+            | D3D11_BIND_CONSTANT_BUFFER
+            | D3D11_BIND_SHADER_RESOURCE
+            | D3D11_BIND_STREAM_OUTPUT
+            | D3D11_BIND_RENDER_TARGET
+            | D3D11_BIND_DEPTH_STENCIL
+            | D3D11_BIND_UNORDERED_ACCESS);
+
+    if (misc_flags & D3D11_RESOURCE_MISC_DRAWINDIRECT_ARGS)
+        wined3d_flags |= WINED3D_BIND_INDIRECT_BUFFER;
+
+    return wined3d_flags;
 }
 
 static inline UINT d3d11_bind_flags_from_wined3d(unsigned int bind_flags)
 {
-    return bind_flags;
+    return bind_flags & (WINED3D_BIND_VERTEX_BUFFER
+            | WINED3D_BIND_INDEX_BUFFER
+            | WINED3D_BIND_CONSTANT_BUFFER
+            | WINED3D_BIND_SHADER_RESOURCE
+            | WINED3D_BIND_STREAM_OUTPUT
+            | WINED3D_BIND_RENDER_TARGET
+            | WINED3D_BIND_DEPTH_STENCIL
+            | WINED3D_BIND_UNORDERED_ACCESS);
 }
 
 /* ID3D11Texture1D, ID3D10Texture1D */
@@ -499,6 +519,7 @@ struct d3d_query *unsafe_impl_from_ID3D11Asynchronous(ID3D11Asynchronous *iface)
 struct d3d11_immediate_context
 {
     ID3D11DeviceContext1 ID3D11DeviceContext1_iface;
+    ID3D11Multithread ID3D11Multithread_iface;
     LONG refcount;
 
     struct wined3d_private_store private_store;
