@@ -18,7 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
 #include <time.h>
 
 #define NONAMELESSUNION
@@ -832,9 +831,7 @@ static unsigned         dump_memory_info(struct dump_context* dc)
         for (pos = 0; pos < dc->mem[i].size; pos += sizeof(tmp))
         {
             len = min(dc->mem[i].size - pos, sizeof(tmp));
-            if (ReadProcessMemory(dc->process->handle,
-                                  (void*)(DWORD_PTR)(dc->mem[i].base + pos),
-                                  tmp, len, NULL))
+            if (read_process_memory(dc->process, dc->mem[i].base + pos, tmp, len))
                 WriteFile(dc->hFile, tmp, len, &written, NULL);
         }
         dc->rva += mdMem.Memory.DataSize;
@@ -890,9 +887,7 @@ static unsigned         dump_memory64_info(struct dump_context* dc)
         for (pos = 0; pos < dc->mem64[i].size; pos += sizeof(tmp))
         {
             len = min(dc->mem64[i].size - pos, sizeof(tmp));
-            if (ReadProcessMemory(dc->process->handle,
-                                  (void*)(ULONG_PTR)(dc->mem64[i].base + pos),
-                                  tmp, len, NULL))
+            if (read_process_memory(dc->process, dc->mem64[i].base + pos, tmp, len))
                 WriteFile(dc->hFile, tmp, len, &written, NULL);
         }
         filepos.QuadPart += mdMem64.DataSize;
