@@ -270,8 +270,8 @@ static void PROFILE_Free( PROFILESECTION *section )
 /* returns TRUE if a whitespace character, else FALSE */
 static inline BOOL PROFILE_isspaceW(WCHAR c)
 {
-	/* ^Z (DOS EOF) is a space too  (found on CD-ROMs) */
-	return isspaceW(c) || c == 0x1a;
+    /* ^Z (DOS EOF) is a space too  (found on CD-ROMs) */
+    return (c >= 0x09 && c <= 0x0d) || c == 0x1a || c == 0x20;
 }
 
 static inline ENCODING PROFILE_DetectTextEncoding(const void * buffer, int * len)
@@ -311,7 +311,7 @@ static PROFILESECTION *PROFILE_Load(HANDLE hFile, ENCODING * pEncoding)
     WCHAR * szFile;
     const WCHAR *szLineStart, *szLineEnd;
     const WCHAR *szValueStart, *szEnd, *next_line;
-    int line = 0, len;
+    int len;
     PROFILESECTION *section, *first_section;
     PROFILESECTION **next_section;
     PROFILEKEY *key, *prev_key, **next_key;
@@ -402,14 +402,10 @@ static PROFILESECTION *PROFILE_Load(HANDLE hFile, ENCODING * pEncoding)
     while (next_line < szEnd)
     {
         szLineStart = next_line;
-        next_line = memchrW(szLineStart, '\n', szEnd - szLineStart);
-        if (!next_line) next_line = memchrW(szLineStart, '\r', szEnd - szLineStart);
-        if (!next_line) next_line = szEnd;
-        else next_line++;
+        while (next_line < szEnd && *next_line != '\n' && *next_line != '\r') next_line++;
+        while (next_line < szEnd && (*next_line == '\n' || *next_line == '\r')) next_line++;
         szLineEnd = next_line;
 
-        line++;
-
         /* get rid of white space */
         while (szLineStart < szLineEnd && PROFILE_isspaceW(*szLineStart)) szLineStart++;
         while ((szLineEnd > szLineStart) && PROFILE_isspaceW(szLineEnd[-1])) szLineEnd--;
@@ -418,16 +414,16 @@ static PROFILESECTION *PROFILE_Load(HANDLE hFile, ENCODING * pEncoding)
 
         if (*szLineStart == '[')  /* section start */
         {
-            const WCHAR * szSectionEnd;
-            if (!(szSectionEnd = memrchrW( szLineStart, ']', szLineEnd - szLineStart )))
+            for (len = szLineEnd - szLineStart; len > 0; len--) if (szLineStart[len - 1] == ']') break;
+            if (!len)
             {
-                WARN("Invalid section header at line %d: %s\n",
-                    line, debugstr_wn(szLineStart, (int)(szLineEnd - szLineStart)) );
+                WARN("Invalid section header: %s\n",
+                    debugstr_wn(szLineStart, (int)(szLineEnd - szLineStart)) );
             }
             else
             {
                 szLineStart++;
-                len = (int)(szSectionEnd - szLineStart);
+                len -= 2;
                 /* no need to allocate +1 for NULL terminating character as
                  * already included in structure */
                 if (!(section = HeapAlloc( GetProcessHeap(), 0, sizeof(*section) + len * sizeof(WCHAR) )))
@@ -450,7 +446,8 @@ static PROFILESECTION *PROFILE_Load(HANDLE hFile, ENCODING * pEncoding)
         /* get rid of white space after the name and before the start
          * of the value */
         len = szLineEnd - szLineStart;
-        if ((szValueStart = memchrW( szLineStart, '=', szLineEnd - szLineStart )) != NULL)
+        for (szValueStart = szLineStart; szValueStart < szLineEnd; szValueStart++) if (*szValueStart == '=') break;
+        if (szValueStart < szLineEnd)
         {
             const WCHAR *szNameEnd = szValueStart;
             while ((szNameEnd > szLineStart) && PROFILE_isspaceW(szNameEnd[-1])) szNameEnd--;
@@ -458,6 +455,7 @@ static PROFILESECTION *PROFILE_Load(HANDLE hFile, ENCODING * pEncoding)
             szValueStart++;
             while (szValueStart < szLineEnd && PROFILE_isspaceW(*szValueStart)) szValueStart++;
         }
+        else szValueStart = NULL;
 
         if (len || !prev_key || *prev_key->name)
         {
@@ -500,7 +498,7 @@ static BOOL PROFILE_DeleteSection( PROFILESECTION **section, LPCWSTR name )
 {
     while (*section)
     {
-        if ((*section)->name[0] && !strcmpiW( (*section)->name, name ))
+        if (!strcmpiW( (*section)->name, name ))
         {
             PROFILESECTION *to_del = *section;
             *section = to_del->next;
@@ -524,7 +522,7 @@ static BOOL PROFILE_DeleteKey( PROFILESECTION **section,
 {
     while (*section)
     {
-        if ((*section)->name[0] && !strcmpiW( (*section)->name, section_name ))
+        if (!strcmpiW( (*section)->name, section_name ))
         {
             PROFILEKEY **key = &(*section)->key;
             while (*key)
@@ -556,7 +554,7 @@ static void PROFILE_DeleteAllKeys( LPCWSTR section_name)
     PROFILESECTION **section= &CurProfile->section;
     while (*section)
     {
-        if ((*section)->name[0] && !strcmpiW( (*section)->name, section_name ))
+        if (!strcmpiW( (*section)->name, section_name ))
         {
             PROFILEKEY **key = &(*section)->key;
             while (*key)
@@ -582,31 +580,28 @@ static PROFILEKEY *PROFILE_Find( PROFILESECTION **section, LPCWSTR section_name,
                                  LPCWSTR key_name, BOOL create, BOOL create_always )
 {
     LPCWSTR p;
-    int seclen, keylen;
+    int seclen = 0, keylen = 0;
 
     while (PROFILE_isspaceW(*section_name)) section_name++;
     if (*section_name)
+    {
         p = section_name + strlenW(section_name) - 1;
-    else
-        p = section_name;
-
-    while ((p > section_name) && PROFILE_isspaceW(*p)) p--;
-    seclen = p - section_name + 1;
+        while ((p > section_name) && PROFILE_isspaceW(*p)) p--;
+        seclen = p - section_name + 1;
+    }
 
     while (PROFILE_isspaceW(*key_name)) key_name++;
     if (*key_name)
+    {
         p = key_name + strlenW(key_name) - 1;
-    else
-        p = key_name;
-
-    while ((p > key_name) && PROFILE_isspaceW(*p)) p--;
-    keylen = p - key_name + 1;
+        while ((p > key_name) && PROFILE_isspaceW(*p)) p--;
+        keylen = p - key_name + 1;
+    }
 
     while (*section)
     {
-        if ( ((*section)->name[0])
-             && (!(strncmpiW( (*section)->name, section_name, seclen )))
-             && (((*section)->name)[seclen] == '\0') )
+        if (!strncmpiW((*section)->name, section_name, seclen) &&
+            ((*section)->name)[seclen] == '\0')
         {
             PROFILEKEY **key = &(*section)->key;
 
@@ -716,15 +711,15 @@ static void PROFILE_ReleaseFile(void)
  * two seconds, so files that are not at least two seconds old might
  * keep their time even on modification, so don't cache them.
  */
-static BOOL is_not_current(FILETIME * ft)
+static BOOL is_not_current(FILETIME *ft)
 {
-    FILETIME Now;
-    LONGLONG ftll, nowll;
-    GetSystemTimeAsFileTime(&Now);
+    LARGE_INTEGER now;
+    LONGLONG ftll;
+
+    NtQuerySystemTime( &now );
     ftll = ((LONGLONG)ft->dwHighDateTime << 32) + ft->dwLowDateTime;
-    nowll = ((LONGLONG)Now.dwHighDateTime << 32) + Now.dwLowDateTime;
-    TRACE("%08x;%08x\n",(unsigned)ftll+21000000,(unsigned)nowll);
-    return ftll + 21000000 < nowll;
+    TRACE("%s; %s\n", wine_dbgstr_longlong(ftll), wine_dbgstr_longlong(now.QuadPart));
+    return ftll + 21000000 < now.QuadPart;
 }
 
 /***********************************************************************
@@ -772,7 +767,7 @@ static BOOL PROFILE_Open( LPCWSTR filename, BOOL write_access )
     else
     {
         LPWSTR dummy;
-        GetFullPathNameW(filename, sizeof(buffer)/sizeof(buffer[0]), buffer, &dummy);
+        GetFullPathNameW(filename, ARRAY_SIZE(buffer), buffer, &dummy);
     }
         
     TRACE("path: %s\n", debugstr_w(buffer));
@@ -873,7 +868,7 @@ static INT PROFILE_GetSection( PROFILESECTION *section, LPCWSTR section_name,
 
     while (section)
     {
-        if (section->name[0] && !strcmpiW( section->name, section_name ))
+        if (!strcmpiW( section->name, section_name ))
         {
             UINT oldlen = len;
             for (key = section->key; key; key = key->next)
@@ -988,11 +983,6 @@ static INT PROFILE_GetString( LPCWSTR section, LPCWSTR key_name,
     if (!def_val) def_val = empty_strW;
     if (key_name)
     {
-	if (!key_name[0])
-        {
-            PROFILE_CopyEntry(buffer, def_val, len, TRUE);
-            return strlenW(buffer);
-        }
         key = PROFILE_Find( &CurProfile->section, section, key_name, FALSE, FALSE);
         PROFILE_CopyEntry( buffer, (key && key->value) ? key->value : def_val,
                            len, TRUE );
@@ -1002,7 +992,7 @@ static INT PROFILE_GetString( LPCWSTR section, LPCWSTR key_name,
         return strlenW( buffer );
     }
     /* no "else" here ! */
-    if (section && section[0])
+    if (section)
     {
         INT ret = PROFILE_GetSection(CurProfile->section, section, buffer, len, FALSE);
         if (!buffer[0]) /* no luck -> def_val */
@@ -1238,9 +1228,8 @@ UINT WINAPI GetPrivateProfileIntW( LPCWSTR section, LPCWSTR entry,
     UNICODE_STRING bufferW;
     ULONG result;
 
-    if (GetPrivateProfileStringW( section, entry, emptystringW,
-                                   buffer, sizeof(buffer)/sizeof(WCHAR),
-                                   filename ) == 0)
+    if (GetPrivateProfileStringW( section, entry, emptystringW, buffer, ARRAY_SIZE( buffer ),
+                                  filename ) == 0)
         return def_val;
 
     /* FIXME: if entry can be found but it's empty, then Win16 is
@@ -1609,6 +1598,23 @@ DWORD WINAPI GetPrivateProfileSectionNamesA( LPSTR buffer, DWORD size,
     return ret;
 }
 
+static int get_hex_byte( const WCHAR *p )
+{
+    int val;
+
+    if (*p >= '0' && *p <= '9') val = *p - '0';
+    else if (*p >= 'A' && *p <= 'Z') val = *p - 'A' + 10;
+    else if (*p >= 'a' && *p <= 'z') val = *p - 'a' + 10;
+    else return -1;
+    val <<= 4;
+    p++;
+    if (*p >= '0' && *p <= '9') val += *p - '0';
+    else if (*p >= 'A' && *p <= 'Z') val += *p - 'A' + 10;
+    else if (*p >= 'a' && *p <= 'z') val += *p - 'a' + 10;
+    else return -1;
+    return val;
+}
+
 /***********************************************************************
  *           GetPrivateProfileStructW (KERNEL32.@)
  *
@@ -1617,70 +1623,29 @@ DWORD WINAPI GetPrivateProfileSectionNamesA( LPSTR buffer, DWORD size,
 BOOL WINAPI GetPrivateProfileStructW (LPCWSTR section, LPCWSTR key,
                                       LPVOID buf, UINT len, LPCWSTR filename)
 {
-    BOOL	ret = FALSE;
+    BOOL ret = FALSE;
+    LPBYTE data = buf;
+    BYTE chksum = 0;
+    int val;
+    WCHAR *p, *buffer;
 
-    RtlEnterCriticalSection( &PROFILE_CritSect );
+    if (!(buffer = HeapAlloc( GetProcessHeap(), 0, (2 * len + 3) * sizeof(WCHAR) ))) return FALSE;
 
-    if (PROFILE_Open( filename, FALSE )) {
-        PROFILEKEY *k = PROFILE_Find ( &CurProfile->section, section, key, FALSE, FALSE);
-	if (k) {
-	    TRACE("value (at %p): %s\n", k->value, debugstr_w(k->value));
-	    if (((strlenW(k->value) - 2) / 2) == len)
-	    {
-		LPWSTR end, p;
-		BOOL valid = TRUE;
-		WCHAR c;
-		DWORD chksum = 0;
+    if (GetPrivateProfileStringW( section, key, NULL, buffer, 2 * len + 3, filename ) != 2 * len + 2)
+        goto done;
 
-	        end  = k->value + strlenW(k->value); /* -> '\0' */
-	        /* check for invalid chars in ASCII coded hex string */
-	        for (p=k->value; p < end; p++)
-		{
-                    if (!isxdigitW(*p))
-		    {
-			WARN("invalid char '%x' in file %s->[%s]->%s !\n",
-                             *p, debugstr_w(filename), debugstr_w(section), debugstr_w(key));
-		        valid = FALSE;
-		        break;
-		    }
-		}
-		if (valid)
-		{
-		    BOOL highnibble = TRUE;
-		    BYTE b = 0, val;
-                    LPBYTE binbuf = buf;
-
-	            end -= 2; /* don't include checksum in output data */
-	            /* translate ASCII hex format into binary data */
-                    for (p=k->value; p < end; p++)
-            	    {
-	        	c = toupperW(*p);
-			val = (c > '9') ?
-				(c - 'A' + 10) : (c - '0');
-
-			if (highnibble)
-		    	    b = val << 4;
-			else
-			{
-		    	    b += val;
-		    	    *binbuf++ = b; /* feed binary data into output */
-		    	    chksum += b; /* calculate checksum */
-			}
-			highnibble ^= 1; /* toggle */
-            	    }
-		    /* retrieve stored checksum value */
-		    c = toupperW(*p++);
-		    b = ( (c > '9') ? (c - 'A' + 10) : (c - '0') ) << 4;
-		    c = toupperW(*p);
-		    b +=  (c > '9') ? (c - 'A' + 10) : (c - '0');
-	            if (b == (chksum & 0xff)) /* checksums match ? */
-                        ret = TRUE;
-                }
-            }
-	}
+    for (p = buffer; len; p += 2, len--)
+    {
+        if ((val = get_hex_byte( p )) == -1) goto done;
+        *data++ = val;
+        chksum += val;
     }
-    RtlLeaveCriticalSection( &PROFILE_CritSect );
+    /* retrieve stored checksum value */
+    if ((val = get_hex_byte( p )) == -1) goto done;
+    ret = ((BYTE)val == chksum);
 
+done:
+    HeapFree( GetProcessHeap(), 0, buffer );
     return ret;
 }
 
@@ -1739,17 +1704,8 @@ BOOL WINAPI WritePrivateProfileStructW (LPCWSTR section, LPCWSTR key,
     *p++ = hex[sum & 0xf];
     *p++ = '\0';
 
-    RtlEnterCriticalSection( &PROFILE_CritSect );
-
-    if (PROFILE_Open( filename, TRUE )) {
-        ret = PROFILE_SetString( section, key, outstring, FALSE);
-        if (ret) ret = PROFILE_FlushFile();
-    }
-
-    RtlLeaveCriticalSection( &PROFILE_CritSect );
-
+    ret = WritePrivateProfileStringW( section, key, outstring, filename );
     HeapFree( GetProcessHeap(), 0, outstring );
-
     return ret;
 }
 
