@@ -104,7 +104,6 @@
 #include "winternl.h"
 #include "ddk/wdm.h"
 #include "ntdll_misc.h"
-#include "wine/unicode.h"
 #include "wine/server.h"
 #include "wine/list.h"
 #include "wine/library.h"
@@ -234,7 +233,7 @@ static inline BOOL is_invalid_dos_char( WCHAR ch )
 {
     static const WCHAR invalid_chars[] = { INVALID_DOS_CHARS,'~','.',0 };
     if (ch > 0x7f) return TRUE;
-    return strchrW( invalid_chars, ch ) != NULL;
+    return wcschr( invalid_chars, ch ) != NULL;
 }
 
 /* check if the device can be a mounted volume */
@@ -305,9 +304,12 @@ static inline unsigned int dir_info_size( FILE_INFORMATION_CLASS class, unsigned
 
 static inline BOOL has_wildcard( const UNICODE_STRING *mask )
 {
-    return (!mask ||
-            memchrW( mask->Buffer, '*', mask->Length / sizeof(WCHAR) ) ||
-            memchrW( mask->Buffer, '?', mask->Length / sizeof(WCHAR) ));
+    int i;
+
+    if (!mask) return TRUE;
+    for (i = 0; i < mask->Length / sizeof(WCHAR); i++)
+        if (mask->Buffer[i] == '*' || mask->Buffer[i] == '?') return TRUE;
+    return FALSE;
 }
 
 /* get space from the current directory data buffer, allocating a new one if necessary */
@@ -344,8 +346,8 @@ static const char *add_dir_data_nameA( struct dir_data *data, const char *name )
 /* add a Unicode string to the directory data buffer */
 static const WCHAR *add_dir_data_nameW( struct dir_data *data, const WCHAR *name )
 {
-    WCHAR *ptr = get_dir_data_space( data, (strlenW( name ) + 1) * sizeof(WCHAR) );
-    if (ptr) strcpyW( ptr, name );
+    WCHAR *ptr = get_dir_data_space( data, (wcslen( name ) + 1) * sizeof(WCHAR) );
+    if (ptr) wcscpy( ptr, name );
     return ptr;
 }
 
@@ -1316,8 +1318,8 @@ static ULONG hash_short_file_name( const UNICODE_STRING *name, LPWSTR buffer )
     if (!is_case_sensitive)
     {
         for (p = name->Buffer, hash = 0xbeef; p < end - 1; p++)
-            hash = (hash<<3) ^ (hash>>5) ^ tolowerW(*p) ^ (tolowerW(p[1]) << 8);
-        hash = (hash<<3) ^ (hash>>5) ^ tolowerW(*p); /* Last character */
+            hash = (hash<<3) ^ (hash>>5) ^ RtlDowncaseUnicodeChar(*p) ^ (RtlDowncaseUnicodeChar(p[1]) << 8);
+        hash = (hash<<3) ^ (hash>>5) ^ RtlDowncaseUnicodeChar(*p); /* Last character */
     }
     else
     {
@@ -1394,7 +1396,7 @@ static BOOLEAN match_filename( const UNICODE_STRING *name_str, const UNICODE_STR
             if (is_case_sensitive)
                 while (name < name_end && (*name != *mask)) name++;
             else
-                while (name < name_end && (toupperW(*name) != toupperW(*mask))) name++;
+                while (name < name_end && (towupper(*name) != towupper(*mask))) name++;
             next_to_retry = name;
             break;
         case '?':
@@ -1403,7 +1405,7 @@ static BOOLEAN match_filename( const UNICODE_STRING *name_str, const UNICODE_STR
             break;
         default:
             if (is_case_sensitive) mismatch = (*mask != *name);
-            else mismatch = (toupperW(*mask) != toupperW(*name));
+            else mismatch = (towupper(*mask) != towupper(*name));
 
             if (!mismatch)
             {
@@ -1519,7 +1521,7 @@ static NTSTATUS get_dir_data_entry( struct dir_data *dir_data, void *info_ptr, I
     if (start + dir_size > max_length) return STATUS_MORE_ENTRIES;
 
     max_length -= start + dir_size;
-    name_len = strlenW( names->long_name ) * sizeof(WCHAR);
+    name_len = wcslen( names->long_name ) * sizeof(WCHAR);
     /* if this is not the first entry, fail; the first entry is always returned (but truncated) */
     if (*last_info && name_len > max_length) return STATUS_MORE_ENTRIES;
 
@@ -1557,14 +1559,14 @@ static NTSTATUS get_dir_data_entry( struct dir_data *dir_data, void *info_ptr, I
 
     case FileBothDirectoryInformation:
         info->both.EaSize = 0; /* FIXME */
-        info->both.ShortNameLength = strlenW( names->short_name ) * sizeof(WCHAR);
+        info->both.ShortNameLength = wcslen( names->short_name ) * sizeof(WCHAR);
         memcpy( info->both.ShortName, names->short_name, info->both.ShortNameLength );
         info->both.FileNameLength = name_len;
         break;
 
     case FileIdBothDirectoryInformation:
         info->id_both.EaSize = 0; /* FIXME */
-        info->id_both.ShortNameLength = strlenW( names->short_name ) * sizeof(WCHAR);
+        info->id_both.ShortNameLength = wcslen( names->short_name ) * sizeof(WCHAR);
         memcpy( info->id_both.ShortName, names->short_name, info->id_both.ShortNameLength );
         info->id_both.FileNameLength = name_len;
         break;
@@ -1815,9 +1817,9 @@ static int name_compare( const void *a, const void *b )
 {
     const struct dir_data_names *file_a = (const struct dir_data_names *)a;
     const struct dir_data_names *file_b = (const struct dir_data_names *)b;
-    int ret = RtlCompareUnicodeStrings( file_a->long_name, strlenW(file_a->long_name),
-                                        file_b->long_name, strlenW(file_b->long_name), TRUE );
-    if (!ret) ret = strcmpW( file_a->long_name, file_b->long_name );
+    int ret = RtlCompareUnicodeStrings( file_a->long_name, wcslen(file_a->long_name),
+                                        file_b->long_name, wcslen(file_b->long_name), TRUE );
+    if (!ret) ret = wcscmp( file_a->long_name, file_b->long_name );
     return ret;
 }
 
@@ -2245,29 +2247,24 @@ static int match_redirect( const WCHAR *path, int len, const WCHAR *redir, BOOLE
 {
     int i = 0;
 
-    while (i < len && *redir)
+    while (i < len)
     {
-        if (IS_SEPARATOR(path[i]))
+        int start = i;
+        while (i < len && !IS_SEPARATOR(path[i])) i++;
+        if (check_case)
         {
-            if (*redir++ != '\\') return 0;
-            while (i < len && IS_SEPARATOR(path[i])) i++;
-            continue;  /* move on to next path component */
-        }
-        else if (check_case)
-        {
-            if (path[i] != *redir) return 0;
+            if (wcsncmp( path + start, redir, i - start )) return 0;
         }
         else
         {
-            if (tolowerW(path[i]) != tolowerW(*redir)) return 0;
+            if (wcsnicmp( path + start, redir, i - start )) return 0;
         }
-        i++;
-        redir++;
+        redir += i - start;
+        while (i < len && IS_SEPARATOR(path[i])) i++;
+        if (!*redir) return i;
+        if (*redir++ != '\\') return 0;
     }
-    if (*redir) return 0;
-    if (i < len && !IS_SEPARATOR(path[i])) return 0;
-    while (i < len && IS_SEPARATOR(path[i])) i++;
-    return i;
+    return 0;
 }
 
 
@@ -2687,7 +2684,7 @@ NTSTATUS nt_to_unix_file_name_attr( const OBJECT_ATTRIBUTES *attr, ANSI_STRING *
 
     /* check for invalid characters */
     for (p = name; p < name + name_len; p++)
-        if (*p < 32 || strchrW( invalid_charsW, *p )) return STATUS_OBJECT_NAME_INVALID;
+        if (*p < 32 || wcschr( invalid_charsW, *p )) return STATUS_OBJECT_NAME_INVALID;
 
     unix_len = name_len * 3 + MAX_DIR_ENTRY_LEN + 3;
     if (!(unix_name = RtlAllocateHeap( GetProcessHeap(), 0, unix_len )))
@@ -2775,7 +2772,7 @@ NTSTATUS CDECL wine_nt_to_unix_file_name( const UNICODE_STRING *nameW, ANSI_STRI
     for (pos = 0; pos < name_len; pos++)
     {
         if (IS_SEPARATOR(name[pos])) break;
-        if (name[pos] < 32 || strchrW( invalid_charsW, name[pos] ))
+        if (name[pos] < 32 || wcschr( invalid_charsW, name[pos] ))
             return STATUS_OBJECT_NAME_INVALID;
     }
     if (pos > MAX_DIR_ENTRY_LEN)
@@ -2803,7 +2800,7 @@ NTSTATUS CDECL wine_nt_to_unix_file_name( const UNICODE_STRING *nameW, ANSI_STRI
     else
     {
         for (p = name; p < name + name_len; p++)
-            if (*p < 32 || strchrW( invalid_charsW, *p )) return STATUS_OBJECT_NAME_INVALID;
+            if (*p < 32 || wcschr( invalid_charsW, *p )) return STATUS_OBJECT_NAME_INVALID;
     }
 
     unix_len = (prefix_len + name_len) * 3 + MAX_DIR_ENTRY_LEN + 3;
