@@ -42,7 +42,6 @@
 #include "winternl.h"
 #include "wine/debug.h"
 #include "wine/exception.h"
-#include "wine/unicode.h"
 #include "ntdll_misc.h"
 #include "inaddr.h"
 #include "in6addr.h"
@@ -1111,15 +1110,15 @@ NTSTATUS WINAPI RtlIpv4AddressToStringExW(const IN_ADDR *pin, USHORT port, LPWST
 
     TRACE("(%p:0x%x, %d, %p, %p:%d)\n", pin, pin->S_un.S_addr, port, buffer, psize, *psize);
 
-    needed = sprintfW(tmp_ip, fmt_ip,
+    needed = NTDLL_swprintf(tmp_ip, fmt_ip,
                       pin->S_un.S_un_b.s_b1, pin->S_un.S_un_b.s_b2,
                       pin->S_un.S_un_b.s_b3, pin->S_un.S_un_b.s_b4);
 
-    if (port) needed += sprintfW(tmp_ip + needed, fmt_port, ntohs(port));
+    if (port) needed += NTDLL_swprintf(tmp_ip + needed, fmt_port, ntohs(port));
 
     if (*psize > needed) {
         *psize = needed + 1;
-        strcpyW(buffer, tmp_ip);
+        wcscpy(buffer, tmp_ip);
         return STATUS_SUCCESS;
     }
 
