@@ -128,15 +128,6 @@ typedef int (*wine_signal_handler)(unsigned int sig);
 static wine_signal_handler handlers[256];
 
 
-struct UNWIND_INFO
-{
-    WORD function_length;
-    WORD unknown1 : 7;
-    WORD count : 5;
-    WORD unknown2 : 4;
-};
-
-
 /***********************************************************************
  *           get_trap_code
  *
@@ -148,7 +139,8 @@ static inline enum arm_trap_code get_trap_code( int signal, const ucontext_t *si
     enum arm_trap_code trap = TRAP_sig(sigcontext);
     if (trap)
         return trap;
-    /* trap is 0 on arm64 kernel */
+#endif
+
     switch (signal)
     {
     case SIGILL:
@@ -158,11 +150,8 @@ static inline enum arm_trap_code get_trap_code( int signal, const ucontext_t *si
     case SIGBUS:
         return TRAP_ARM_ALIGNFLT;
     default:
-        return trap;
+        return TRAP_ARM_UNKNOWN;
     }
-#else
-    return TRAP_ARM_UNKNOWN;  /* unknown trap code */
-#endif
 }
 
 /***********************************************************************
@@ -732,18 +721,17 @@ static void segv_handler( int signal, siginfo_t *info, void *ucontext )
     ucontext_t *context = ucontext;
 
     /* check for page fault inside the thread stack */
-    if (get_trap_code(signal, context) == TRAP_ARM_PAGEFLT &&
-        (char *)info->si_addr >= (char *)NtCurrentTeb()->DeallocationStack &&
-        (char *)info->si_addr < (char *)NtCurrentTeb()->Tib.StackBase &&
-        virtual_handle_stack_fault( info->si_addr ))
+    if (get_trap_code(signal, context) == TRAP_ARM_PAGEFLT)
     {
-        /* check if this was the last guard page */
-        if ((char *)info->si_addr < (char *)NtCurrentTeb()->DeallocationStack + 2*4096)
+        switch (virtual_handle_stack_fault( info->si_addr ))
         {
+        case 1:  /* handled */
+            return;
+        case -1:  /* overflow */
             rec = setup_exception( context, raise_segv_exception );
             rec->ExceptionCode = EXCEPTION_STACK_OVERFLOW;
+            return;
         }
-        return;
     }
 
     rec = setup_exception( context, raise_segv_exception );
@@ -969,22 +957,22 @@ int CDECL __wine_set_signal_handler(unsigned int sig, wine_signal_handler wsh)
  */
 NTSTATUS signal_alloc_thread( TEB **teb )
 {
-    static size_t sigstack_zero_bits;
+    static size_t sigstack_alignment;
     SIZE_T size;
     NTSTATUS status;
 
-    if (!sigstack_zero_bits)
+    if (!sigstack_alignment)
     {
         size_t min_size = page_size;
         /* find the first power of two not smaller than min_size */
-        while ((1u << sigstack_zero_bits) < min_size) sigstack_zero_bits++;
+        while ((1u << sigstack_alignment) < min_size) sigstack_alignment++;
         assert( sizeof(TEB) <= min_size );
     }
 
-    size = 1 << sigstack_zero_bits;
+    size = 1 << sigstack_alignment;
     *teb = NULL;
-    if (!(status = NtAllocateVirtualMemory( NtCurrentProcess(), (void **)teb, sigstack_zero_bits,
-                                            &size, MEM_COMMIT | MEM_TOP_DOWN, PAGE_READWRITE )))
+    if (!(status = virtual_alloc_aligned( (void **)teb, 0, &size, MEM_COMMIT | MEM_TOP_DOWN,
+                                          PAGE_READWRITE, sigstack_alignment )))
     {
         (*teb)->Tib.Self = &(*teb)->Tib;
         (*teb)->Tib.ExceptionList = (void *)~0UL;
@@ -1066,118 +1054,6 @@ void signal_init_process(void)
 }
 
 
-/**********************************************************************
- *              RtlAddFunctionTable   (NTDLL.@)
- */
-BOOLEAN CDECL RtlAddFunctionTable( RUNTIME_FUNCTION *table, DWORD count, DWORD addr )
-{
-    FIXME( "%p %u %x: stub\n", table, count, addr );
-    return TRUE;
-}
-
-/**********************************************************************
- *              RtlInstallFunctionTableCallback   (NTDLL.@)
- */
-BOOLEAN CDECL RtlInstallFunctionTableCallback( DWORD table, DWORD base, DWORD length,
-                                               PGET_RUNTIME_FUNCTION_CALLBACK callback, PVOID context, PCWSTR dll )
-{
-    FIXME( "%x %x %d %p %p %s: stub\n", table, base, length, callback, context, wine_dbgstr_w(dll) );
-    return TRUE;
-}
-
-/*************************************************************************
- *              RtlAddGrowableFunctionTable   (NTDLL.@)
- */
-DWORD WINAPI RtlAddGrowableFunctionTable( void **table, RUNTIME_FUNCTION *functions, DWORD count, DWORD max_count,
-                                          ULONG_PTR base, ULONG_PTR end )
-{
-    FIXME( "(%p, %p, %d, %d, %ld, %ld) stub!\n", table, functions, count, max_count, base, end );
-    if (table) *table = NULL;
-    return STATUS_SUCCESS;
-}
-
-/*************************************************************************
- *              RtlGrowFunctionTable   (NTDLL.@)
- */
-void WINAPI RtlGrowFunctionTable( void *table, DWORD count )
-{
-    FIXME( "(%p, %d) stub!\n", table, count );
-}
-
-/*************************************************************************
- *              RtlDeleteGrowableFunctionTable   (NTDLL.@)
- */
-void WINAPI RtlDeleteGrowableFunctionTable( void *table )
-{
-    FIXME( "(%p) stub!\n", table );
-}
-
-/**********************************************************************
- *              RtlDeleteFunctionTable   (NTDLL.@)
- */
-BOOLEAN CDECL RtlDeleteFunctionTable( RUNTIME_FUNCTION *table )
-{
-    FIXME( "%p: stub\n", table );
-    return TRUE;
-}
-
-/**********************************************************************
- *              find_function_info
- */
-static RUNTIME_FUNCTION *find_function_info( ULONG_PTR pc, HMODULE module,
-                                             RUNTIME_FUNCTION *func, ULONG size )
-{
-    int min = 0;
-    int max = size/sizeof(*func) - 1;
-
-    while (min <= max)
-    {
-        int pos = (min + max) / 2;
-        DWORD begin = (func[pos].BeginAddress & ~1), end;
-        if (func[pos].u.s.Flag)
-            end = begin + func[pos].u.s.FunctionLength * 2;
-        else
-        {
-            struct UNWIND_INFO *info;
-            info = (struct UNWIND_INFO *)((char *)module + func[pos].u.UnwindData);
-            end = begin + info->function_length * 2;
-        }
-
-        if ((char *)pc < (char *)module + begin) max = pos - 1;
-        else if ((char *)pc >= (char *)module + end) min = pos + 1;
-        else return func + pos;
-    }
-    return NULL;
-}
-
-/**********************************************************************
- *              RtlLookupFunctionEntry   (NTDLL.@)
- */
-PRUNTIME_FUNCTION WINAPI RtlLookupFunctionEntry( ULONG_PTR pc, DWORD *base,
-                                                 UNWIND_HISTORY_TABLE *table )
-{
-    LDR_MODULE *module;
-    RUNTIME_FUNCTION *func;
-    ULONG size;
-
-    /* FIXME: should use the history table to make things faster */
-
-    if (LdrFindEntryForAddress( (void *)pc, &module ))
-    {
-        WARN( "module not found for %lx\n", pc );
-        return NULL;
-    }
-    if (!(func = RtlImageDirectoryEntryToData( module->BaseAddress, TRUE,
-                                               IMAGE_DIRECTORY_ENTRY_EXCEPTION, &size )))
-    {
-        WARN( "no exception table found in module %p pc %lx\n", module->BaseAddress, pc );
-        return NULL;
-    }
-    func = find_function_info( pc, module->BaseAddress, func, size );
-    if (func) *base = (DWORD)module->BaseAddress;
-    return func;
-}
-
 /***********************************************************************
  *            RtlUnwind  (NTDLL.@)
  */
@@ -1401,6 +1277,23 @@ void signal_exit_process( int status )
     call_thread_exit_func( status, exit, NtCurrentTeb() );
 }
 
+/**********************************************************************
+ *           get_thread_ldt_entry
+ */
+NTSTATUS get_thread_ldt_entry( HANDLE handle, void *data, ULONG len, ULONG *ret_len )
+{
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+/******************************************************************************
+ *           NtSetLdtEntries   (NTDLL.@)
+ *           ZwSetLdtEntries   (NTDLL.@)
+ */
+NTSTATUS WINAPI NtSetLdtEntries( ULONG sel1, LDT_ENTRY entry1, ULONG sel2, LDT_ENTRY entry2 )
+{
+    return STATUS_NOT_IMPLEMENTED;
+}
+
 /**********************************************************************
  *              DbgBreakPoint   (NTDLL.@)
  */
