@@ -31,7 +31,6 @@
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
-#include "wine/unicode.h"
 #include "wine/debug.h"
 #include "ntdll_misc.h"
 #include "ddk/wdm.h"
@@ -281,13 +280,13 @@ static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
     {
         WCHAR *p, *str = (WCHAR *)info->Data;
         str[info->DataLength / sizeof(WCHAR)] = 0;
-        p = strchrW( str, '.' );
+        p = wcschr( str, '.' );
         if (p)
         {
             *p++ = 0;
-            version->dwMinorVersion = atoiW( p );
+            version->dwMinorVersion = wcstoul( p, NULL, 10 );
         }
-        version->dwMajorVersion = atoiW( str );
+        version->dwMajorVersion = wcstoul( str, NULL, 10 );
     }
 
     if (version->dwMajorVersion)   /* we got the main version, now fetch the other fields */
@@ -302,7 +301,7 @@ static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
         {
             WCHAR *str = (WCHAR *)info->Data;
             str[info->DataLength / sizeof(WCHAR)] = 0;
-            version->dwBuildNumber = atoiW( str );
+            version->dwBuildNumber = wcstoul( str, NULL, 10 );
         }
 
         /* get version description */
@@ -398,19 +397,19 @@ static BOOL get_win9x_registry_version( RTL_OSVERSIONINFOEXW *version )
     {
         WCHAR *p, *str = (WCHAR *)info->Data;
         str[info->DataLength / sizeof(WCHAR)] = 0;
-        p = strchrW( str, '.' );
+        p = wcschr( str, '.' );
         if (p) *p++ = 0;
-        version->dwMajorVersion = atoiW( str );
+        version->dwMajorVersion = wcstoul( str, NULL, 10 );
         if (p)
         {
             str = p;
-            p = strchrW( str, '.' );
+            p = wcschr( str, '.' );
             if (p)
             {
                 *p++ = 0;
-                version->dwBuildNumber = atoiW( p );
+                version->dwBuildNumber = wcstoul( p, NULL, 10 );
             }
-            version->dwMinorVersion = atoiW( str );
+            version->dwMinorVersion = wcstoul( str, NULL, 10 );
         }
         /* build number contains version too on Win9x */
         version->dwBuildNumber |= MAKEWORD( version->dwMinorVersion, version->dwMajorVersion ) << 16;
@@ -458,7 +457,7 @@ static BOOL parse_win_version( HANDLE hkey )
 
     for (i = 0; i < ARRAY_SIZE(version_names); i++)
     {
-        if (strcmpW( version_names[i].name, name )) continue;
+        if (wcscmp( version_names[i].name, name )) continue;
         current_version = &VersionData[version_names[i].ver];
         TRACE( "got win version %s\n", debugstr_w(version_names[i].name) );
         return TRUE;
@@ -501,11 +500,11 @@ void version_init(void)
 
     /* open AppDefaults\\appname key */
 
-    if ((p = strrchrW( appname, '/' ))) appname = p + 1;
-    if ((p = strrchrW( appname, '\\' ))) appname = p + 1;
+    if ((p = wcsrchr( appname, '/' ))) appname = p + 1;
+    if ((p = wcsrchr( appname, '\\' ))) appname = p + 1;
 
-    strcpyW( appversion, appdefaultsW );
-    strcatW( appversion, appname );
+    wcscpy( appversion, appdefaultsW );
+    wcscat( appversion, appname );
     RtlInitUnicodeString( &nameW, appversion );
     attr.RootDirectory = config_key;
 
@@ -595,7 +594,7 @@ NTSTATUS WINAPI RtlGetVersion( RTL_OSVERSIONINFOEXW *info )
     info->dwMinorVersion = current_version->dwMinorVersion;
     info->dwBuildNumber  = current_version->dwBuildNumber;
     info->dwPlatformId   = current_version->dwPlatformId;
-    strcpyW( info->szCSDVersion, current_version->szCSDVersion );
+    wcscpy( info->szCSDVersion, current_version->szCSDVersion );
     if(info->dwOSVersionInfoSize == sizeof(RTL_OSVERSIONINFOEXW))
     {
         info->wServicePackMajor = current_version->wServicePackMajor;
