@@ -1365,11 +1365,9 @@ static void test_AviMux(char *arg)
     VIDEOINFO videoinfo;
     IPin *avimux_in, *avimux_out, *pin;
     AM_MEDIA_TYPE source_media_type;
-    AM_MEDIA_TYPE *media_type;
     PIN_DIRECTION dir;
     IBaseFilter *avimux;
     IEnumPins *ep;
-    IEnumMediaTypes *emt;
     IMemInputPin *memin;
     ALLOCATOR_PROPERTIES props;
     IMemAllocator *memalloc;
@@ -1405,35 +1403,6 @@ static void test_AviMux(char *arg)
     ok(dir == PINDIR_INPUT, "dir = %d\n", dir);
     IEnumPins_Release(ep);
 
-    hr = IPin_EnumMediaTypes(avimux_out, &emt);
-    ok(hr == S_OK, "EnumMediaTypes returned %x\n", hr);
-    hr = IEnumMediaTypes_Next(emt, 1, &media_type, NULL);
-    ok(hr == S_OK, "Next returned %x\n", hr);
-    ok(IsEqualIID(&media_type->majortype, &MEDIATYPE_Stream), "majortype = %s\n",
-            wine_dbgstr_guid(&media_type->majortype));
-    ok(IsEqualIID(&media_type->subtype, &MEDIASUBTYPE_Avi), "subtype = %s\n",
-            wine_dbgstr_guid(&media_type->subtype));
-    ok(media_type->bFixedSizeSamples, "bFixedSizeSamples = %x\n", media_type->bFixedSizeSamples);
-    ok(!media_type->bTemporalCompression, "bTemporalCompression = %x\n", media_type->bTemporalCompression);
-    ok(media_type->lSampleSize == 1, "lSampleSize = %d\n", media_type->lSampleSize);
-    ok(IsEqualIID(&media_type->formattype, &GUID_NULL), "formattype = %s\n",
-            wine_dbgstr_guid(&media_type->formattype));
-    ok(!media_type->pUnk, "pUnk = %p\n", media_type->pUnk);
-    ok(!media_type->cbFormat, "cbFormat = %d\n", media_type->cbFormat);
-    ok(!media_type->pbFormat, "pbFormat = %p\n", media_type->pbFormat);
-    CoTaskMemFree(media_type);
-    hr = IEnumMediaTypes_Next(emt, 1, &media_type, NULL);
-    ok(hr == S_FALSE, "Next returned %x\n", hr);
-    IEnumMediaTypes_Release(emt);
-
-    hr = IPin_EnumMediaTypes(avimux_in, &emt);
-    ok(hr == S_OK, "EnumMediaTypes returned %x\n", hr);
-    hr = IEnumMediaTypes_Reset(emt);
-    ok(hr == S_OK, "Reset returned %x\n", hr);
-    hr = IEnumMediaTypes_Next(emt, 1, &media_type, NULL);
-    ok(hr == S_FALSE, "Next returned %x\n", hr);
-    IEnumMediaTypes_Release(emt);
-
     hr = IPin_ReceiveConnection(avimux_in, &source_filter.IPin_iface, NULL);
     ok(hr == E_POINTER, "ReceiveConnection returned %x\n", hr);
 
@@ -1464,7 +1433,7 @@ static void test_AviMux(char *arg)
             pin, &source_filter.IPin_iface);
 
     hr = IPin_Connect(avimux_out, &source_filter.IPin_iface, NULL);
-    todo_wine ok(hr == VFW_E_INVALID_DIRECTION, "Connect returned %x\n", hr);
+    ok(hr == VFW_E_INVALID_DIRECTION, "Connect returned %x\n", hr);
 
     hr = IBaseFilter_JoinFilterGraph(avimux, (IFilterGraph*)&GraphBuilder, NULL);
     ok(hr == S_OK, "JoinFilterGraph returned %x\n", hr);
@@ -1809,7 +1778,7 @@ static void test_COM_vfwcapture(void)
 
     /* Unsupported interfaces */
     hr = IBaseFilter_QueryInterface(bf, &IID_IAMStreamConfig, (void**)&unk);
-    todo_wine ok(hr == E_NOINTERFACE, "QueryInterface for IID_IAMStreamConfig failed: %08x\n", hr);
+    ok(hr == E_NOINTERFACE, "QueryInterface for IID_IAMStreamConfig failed: %08x\n", hr);
     hr = IBaseFilter_QueryInterface(bf, &IID_IAMVideoProcAmp, (void**)&unk);
     todo_wine ok(hr == E_NOINTERFACE, "QueryInterface for IID_IAMVideoProcAmp failed: %08x\n", hr);
     hr = IBaseFilter_QueryInterface(bf, &IID_IOverlayNotify, (void**)&unk);
