@@ -30,7 +30,6 @@
 #include <CoreServices/CoreServices.h>
 #undef GetCurrentThread
 #undef LoadResource
-#undef DPRINTF
 #endif
 
 #include "windef.h"
@@ -39,7 +38,6 @@
 #include "schannel.h"
 #include "secur32_priv.h"
 #include "wine/debug.h"
-#include "wine/library.h"
 
 #ifdef HAVE_SECURITY_SECURITY_H
 
@@ -1203,7 +1201,7 @@ BOOL schan_imp_init(void)
     supported_protocols = SP_PROT_SSL2_CLIENT | SP_PROT_SSL3_CLIENT | SP_PROT_TLS1_0_CLIENT;
 
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
-    if(SSLGetProtocolVersionMax != NULL) {
+    if(&SSLGetProtocolVersionMax != NULL) {
         SSLProtocol max_protocol;
         SSLContextRef ctx;
         OSStatus status;
