@@ -146,7 +146,6 @@ static void test_WsCreateServiceProxyFromTemplate(void)
 
 static void test_WsOpenServiceProxy(void)
 {
-    WCHAR url[] = {'h','t','t','p',':','/','/','l','o','c','a','l','h','o','s','t','/'};
     HRESULT hr;
     WS_SERVICE_PROXY *proxy;
     WS_SERVICE_PROXY_STATE state;
@@ -164,8 +163,8 @@ static void test_WsOpenServiceProxy(void)
     ok( state == WS_SERVICE_PROXY_STATE_CREATED, "got %u\n", state );
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = ARRAY_SIZE( url );
-    addr.url.chars  = url;
+    addr.url.length = ARRAY_SIZE( L"http://localhost/" ) - 1;
+    addr.url.chars  = (WCHAR *)L"http://localhost/";
     hr = WsOpenServiceProxy( proxy, &addr, NULL, NULL );
     ok( hr == S_OK, "got %08x\n", hr );
 
@@ -187,7 +186,6 @@ static void test_WsOpenServiceProxy(void)
 
 static void test_WsResetServiceProxy(void)
 {
-    WCHAR url[] = {'h','t','t','p',':','/','/','l','o','c','a','l','h','o','s','t','/'};
     HRESULT hr;
     WS_SERVICE_PROXY *proxy;
     WS_ENDPOINT_ADDRESS addr;
@@ -206,8 +204,8 @@ static void test_WsResetServiceProxy(void)
     ok( state == WS_SERVICE_PROXY_STATE_CREATED, "got %u\n", state );
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = ARRAY_SIZE( url );
-    addr.url.chars  = url;
+    addr.url.length = ARRAY_SIZE( L"http://localhost/" ) - 1;
+    addr.url.chars  = (WCHAR *)L"http://localhost/";
     hr = WsOpenServiceProxy( proxy, &addr, NULL, NULL );
     ok( hr == S_OK, "got %08x\n", hr );
 
@@ -230,8 +228,6 @@ static void test_WsResetServiceProxy(void)
 
 static HRESULT create_channel( int port, WS_CHANNEL **ret )
 {
-    static const WCHAR fmt[] =
-        {'h','t','t','p',':','/','/','1','2','7','.','0','.','0','.','1',':','%','u',0};
     WS_CHANNEL_PROPERTY prop[2];
     WS_ENVELOPE_VERSION env_version = WS_ENVELOPE_VERSION_SOAP_1_1;
     WS_ADDRESSING_VERSION addr_version = WS_ADDRESSING_VERSION_TRANSPORT;
@@ -253,7 +249,7 @@ static HRESULT create_channel( int port, WS_CHANNEL **ret )
     if (hr != S_OK) return hr;
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = wsprintfW( buf, fmt, port );
+    addr.url.length = wsprintfW( buf, L"http://127.0.0.1:%u", port );
     addr.url.chars  = buf;
     hr = WsOpenChannel( channel, &addr, NULL, NULL );
     if (hr == S_OK) *ret = channel;
@@ -361,13 +357,32 @@ static void test_WsReceiveMessage( int port )
     WsFreeMessage( msg );
 }
 
+static WS_HTTP_HEADER_MAPPING mapped_request_header =
+{
+    {19, (BYTE *)"MappedRequestHeader"}
+};
+
+static WS_HTTP_HEADER_MAPPING *request_header_mappings[] =
+{
+    &mapped_request_header
+};
+
+static WS_HTTP_HEADER_MAPPING mapped_response_header =
+{
+    {20, (BYTE *)"MappedResponseHeader"}
+};
+
+static WS_HTTP_HEADER_MAPPING *response_header_mappings[] =
+{
+    &mapped_response_header
+};
+
 static HRESULT create_proxy( int port, WS_SERVICE_PROXY **ret )
 {
-    static const WCHAR fmt[] =
-        {'h','t','t','p',':','/','/','1','2','7','.','0','.','0','.','1',':','%','u','/',0};
     WS_ENVELOPE_VERSION env_version;
     WS_ADDRESSING_VERSION addr_version;
-    WS_CHANNEL_PROPERTY prop[2];
+    WS_HTTP_MESSAGE_MAPPING mapping;
+    WS_CHANNEL_PROPERTY prop[3];
     WS_ENDPOINT_ADDRESS addr;
     WS_SERVICE_PROXY *proxy;
     WCHAR url[64];
@@ -383,13 +398,24 @@ static HRESULT create_proxy( int port, WS_SERVICE_PROXY **ret )
     prop[1].value     = &addr_version;
     prop[1].valueSize = sizeof(addr_version);
 
+    mapping.requestMappingOptions      = 0;
+    mapping.responseMappingOptions     = 0;
+    mapping.requestHeaderMappings      = request_header_mappings;
+    mapping.requestHeaderMappingCount  = ARRAY_SIZE(request_header_mappings);
+    mapping.responseHeaderMappings     = response_header_mappings;
+    mapping.responseHeaderMappingCount = ARRAY_SIZE(response_header_mappings);
+
+    prop[2].id        = WS_CHANNEL_PROPERTY_HTTP_MESSAGE_MAPPING;
+    prop[2].value     = &mapping;
+    prop[2].valueSize = sizeof(mapping);
+
     *ret = NULL;
     hr = WsCreateServiceProxy( WS_CHANNEL_TYPE_REQUEST, WS_HTTP_CHANNEL_BINDING, NULL, NULL,
                                0, prop, ARRAY_SIZE( prop ), &proxy, NULL );
     if (hr != S_OK) return hr;
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = wsprintfW( url, fmt, port );
+    addr.url.length = wsprintfW( url, L"http://127.0.0.1:%u", port );
     addr.url.chars  = url;
     hr = WsOpenServiceProxy( proxy, &addr, NULL, NULL );
     if (hr == S_OK) *ret = proxy;
@@ -397,6 +423,38 @@ static HRESULT create_proxy( int port, WS_SERVICE_PROXY **ret )
     return hr;
 }
 
+static HRESULT set_output( WS_XML_WRITER *writer )
+{
+    WS_XML_WRITER_TEXT_ENCODING text = {{ WS_XML_WRITER_ENCODING_TYPE_TEXT }, WS_CHARSET_UTF8 };
+    WS_XML_WRITER_BUFFER_OUTPUT buf = {{ WS_XML_WRITER_OUTPUT_TYPE_BUFFER }};
+    return WsSetOutput( writer, &text.encoding, &buf.output, NULL, 0, NULL );
+}
+
+static void check_output_headers( WS_MESSAGE *msg )
+{
+    WS_XML_WRITER *writer;
+    WS_XML_BUFFER *buf;
+    WS_BYTES bytes;
+    HRESULT hr;
+
+    hr = WsCreateWriter( NULL, 0, &writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsGetMessageProperty( msg, WS_MESSAGE_PROPERTY_HEADER_BUFFER, &buf, sizeof(buf), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteXmlBuffer( writer, buf, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    memset( &bytes, 0, sizeof(bytes) );
+    hr = WsGetWriterProperty( writer, WS_XML_WRITER_PROPERTY_BYTES, &bytes, sizeof(bytes), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    WsFreeWriter( writer );
+}
+
 static const char req_test2[] =
     "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"><s:Body>"
     "<req_test2 xmlns=\"ns\"><val>1</val><str>test</str><str>test2</str></req_test2>"
@@ -407,9 +465,33 @@ static const char resp_test2[] =
     "<resp_test2 xmlns=\"ns\"><str>test</str><val>1</val><val>2</val></resp_test2>"
     "</s:Body></s:Envelope>";
 
+static HRESULT CALLBACK send_callback( WS_MESSAGE *msg, WS_HEAP *heap, void *state, WS_ERROR *error )
+{
+    static const WS_XML_STRING header = {19, (BYTE *)"MappedRequestHeader"}, value = {5, (BYTE *)"value"};
+    HRESULT hr;
+
+    hr = WsAddMappedHeader( msg, &header, WS_XML_STRING_TYPE, WS_WRITE_REQUIRED_VALUE, &value, sizeof(value), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output_headers( msg );
+    return S_OK;
+}
+
+static HRESULT CALLBACK recv_callback( WS_MESSAGE *msg, WS_HEAP *heap, void *state, WS_ERROR *error )
+{
+    static const WS_XML_STRING header = {20, (BYTE *)"MappedResponseHeader"};
+    WCHAR *str;
+    HRESULT hr;
+
+    check_output_headers( msg );
+    hr = WsGetMappedHeader( msg, &header, WS_SINGLETON_HEADER, 0, WS_WSZ_TYPE, WS_READ_OPTIONAL_POINTER, heap,
+                            &str, sizeof(str), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    ok( !wcscmp(str, L"value"), "wrong value %s\n", wine_dbgstr_w(str) );
+    return S_OK;
+}
+
 static void test_WsCall( int port )
 {
-    static const WCHAR testW[] = {'t','e','s','t',0}, test2W[] = {'t','e','s','t','2',0};
     WS_XML_STRING str = {3, (BYTE *)"str"};
     WS_XML_STRING req = {3, (BYTE *)"req"};
     WS_XML_STRING resp = {4, (BYTE *)"resp"};
@@ -421,6 +503,9 @@ static void test_WsCall( int port )
     WS_XML_STRING ns = {2, (BYTE *)"ns"};
     HRESULT hr;
     WS_SERVICE_PROXY *proxy;
+    WS_PROXY_MESSAGE_CALLBACK_CONTEXT ctx_send;
+    WS_PROXY_MESSAGE_CALLBACK_CONTEXT ctx_recv;
+    WS_CALL_PROPERTY prop[2];
     WS_OPERATION_DESCRIPTION op;
     WS_MESSAGE_DESCRIPTION input_msg, output_msg;
     WS_ELEMENT_DESCRIPTION input_elem, output_elem;
@@ -491,8 +576,8 @@ static void test_WsCall( int port )
     ok( hr == E_INVALIDARG, "got %08x\n", hr );
 
     in.val   = 1;
-    str_array[0] = testW;
-    str_array[1] = test2W;
+    str_array[0] = L"test";
+    str_array[1] = L"test2";
     in.str   = str_array;
     in.count = 2;
 
@@ -511,9 +596,23 @@ static void test_WsCall( int port )
     args[4] = &val_ptr;
     args[5] = &count_ptr;
 
-    hr = WsCall( proxy, &op, args, heap, NULL, 0, NULL, NULL );
+    ctx_send.callback = send_callback;
+    ctx_send.state    = NULL;
+
+    prop[0].id        = WS_CALL_PROPERTY_SEND_MESSAGE_CONTEXT;
+    prop[0].value     = &ctx_send;
+    prop[0].valueSize = sizeof(ctx_send);
+
+    ctx_recv.callback = recv_callback;
+    ctx_recv.state    = NULL;
+
+    prop[1].id        = WS_CALL_PROPERTY_RECEIVE_MESSAGE_CONTEXT;
+    prop[1].value     = &ctx_recv;
+    prop[1].valueSize = sizeof(ctx_recv);
+
+    hr = WsCall( proxy, &op, args, heap, prop, ARRAY_SIZE(prop), NULL, NULL );
     ok( hr == S_OK, "got %08x\n", hr );
-    ok( !lstrcmpW( out.str, testW ), "wrong data\n" );
+    ok( !wcscmp( out.str, L"test" ), "wrong data\n" );
     ok( out.count == 2, "got %u\n", out.count );
     ok( out.val[0] == 1, "got %u\n", out.val[0] );
     ok( out.val[1] == 2, "got %u\n", out.val[1] );
@@ -616,7 +715,7 @@ tests[] =
 static void send_response( int c, const char *status, const char *data, unsigned int len )
 {
     static const char headers[] =
-        "Content-Type: text/xml; charset=utf-8\r\nConnection: close\r\n";
+        "Content-Type: text/xml; charset=utf-8\r\nConnection: close\r\nMappedResponseHeader: value\r\n";
     static const char fmt[] =
         "Content-Length: %u\r\n\r\n";
     char buf[128];
@@ -694,10 +793,11 @@ static DWORD CALLBACK server_proc( void *arg )
                 if (tests[j].req_data)
                 {
                     int data_len = strlen( buf );
-                    ok( tests[j].req_len == data_len, "%u: unexpected data length %u %u\n",
+                    ok( tests[j].req_len == data_len, "%u: got data length %u expected %u\n",
                         j, data_len, tests[j].req_len );
                     if (tests[j].req_len == data_len)
-                        ok( !memcmp( tests[j].req_data, buf, tests[j].req_len ), "%u: unexpected data %s\n", j, buf );
+                        ok( !memcmp( tests[j].req_data, buf, tests[j].req_len ),
+                            "%u: got data '%s' expected '%s'\n", j, buf, tests[j].req_data );
                 }
                 send_response( c, tests[j].resp_status, tests[j].resp_data, tests[j].resp_len );
                 break;
