@@ -395,7 +395,7 @@ void CDECL wined3d_resource_preload(struct wined3d_resource *resource)
     wined3d_cs_emit_preload_resource(resource->device->cs, resource);
 }
 
-BOOL wined3d_resource_allocate_sysmem(struct wined3d_resource *resource)
+static BOOL wined3d_resource_allocate_sysmem(struct wined3d_resource *resource)
 {
     void **p;
     SIZE_T align = RESOURCE_ALIGNMENT - 1 + sizeof(*p);
@@ -415,6 +415,14 @@ BOOL wined3d_resource_allocate_sysmem(struct wined3d_resource *resource)
     return TRUE;
 }
 
+BOOL wined3d_resource_prepare_sysmem(struct wined3d_resource *resource)
+{
+    if (resource->heap_memory)
+        return TRUE;
+
+    return wined3d_resource_allocate_sysmem(resource);
+}
+
 void wined3d_resource_free_sysmem(struct wined3d_resource *resource)
 {
     void **p = resource->heap_memory;
@@ -481,17 +489,15 @@ BOOL wined3d_resource_is_offscreen(struct wined3d_resource *resource)
 
 void wined3d_resource_update_draw_binding(struct wined3d_resource *resource)
 {
+    const struct wined3d_d3d_info *d3d_info = &resource->device->adapter->d3d_info;
+
     if (!wined3d_resource_is_offscreen(resource) || wined3d_settings.offscreen_rendering_mode != ORM_FBO)
     {
         resource->draw_binding = WINED3D_LOCATION_DRAWABLE;
     }
     else if (resource->multisample_type)
     {
-        const struct wined3d_gl_info *gl_info = &resource->device->adapter->gl_info;
-        if (gl_info->supported[ARB_TEXTURE_MULTISAMPLE])
-            resource->draw_binding = WINED3D_LOCATION_TEXTURE_RGB;
-        else
-            resource->draw_binding = WINED3D_LOCATION_RB_MULTISAMPLE;
+        resource->draw_binding = d3d_info->multisample_draw_location;
     }
     else if (resource->gl_type == WINED3D_GL_RES_TYPE_RB)
     {
@@ -511,3 +517,61 @@ const struct wined3d_format *wined3d_resource_get_decompress_format(const struct
         return wined3d_get_format(adapter, WINED3DFMT_B8G8R8A8_UNORM_SRGB, resource->bind_flags);
     return wined3d_get_format(adapter, WINED3DFMT_B8G8R8A8_UNORM, resource->bind_flags);
 }
+
+unsigned int wined3d_resource_get_sample_count(const struct wined3d_resource *resource)
+{
+    const struct wined3d_format *format = resource->format;
+
+    /* TODO: NVIDIA expose their Coverage Sample Anti-Aliasing (CSAA)
+     * feature through type == MULTISAMPLE_XX and quality != 0. This could
+     * be mapped to GL_NV_framebuffer_multisample_coverage.
+     *
+     * AMD have a similar feature called Enhanced Quality Anti-Aliasing
+     * (EQAA), but it does not have an equivalent OpenGL extension. */
+
+    /* We advertise as many WINED3D_MULTISAMPLE_NON_MASKABLE quality
+     * levels as the count of advertised multisample types for the texture
+     * format. */
+    if (resource->multisample_type == WINED3D_MULTISAMPLE_NON_MASKABLE)
+    {
+        unsigned int i, count = 0;
+
+        for (i = 0; i < sizeof(format->multisample_types) * CHAR_BIT; ++i)
+        {
+            if (format->multisample_types & 1u << i)
+            {
+                if (resource->multisample_quality == count++)
+                    break;
+            }
+        }
+        return i + 1;
+    }
+
+    return resource->multisample_type;
+}
+
+VkAccessFlags vk_access_mask_from_bind_flags(uint32_t bind_flags)
+{
+    VkAccessFlags flags = 0;
+
+    if (bind_flags & WINED3D_BIND_VERTEX_BUFFER)
+        flags |= VK_ACCESS_VERTEX_ATTRIBUTE_READ_BIT;
+    if (bind_flags & WINED3D_BIND_INDEX_BUFFER)
+        flags |= VK_ACCESS_INDEX_READ_BIT;
+    if (bind_flags & WINED3D_BIND_CONSTANT_BUFFER)
+        flags |= VK_ACCESS_UNIFORM_READ_BIT;
+    if (bind_flags & WINED3D_BIND_SHADER_RESOURCE)
+        flags |= VK_ACCESS_SHADER_READ_BIT;
+    if (bind_flags & WINED3D_BIND_UNORDERED_ACCESS)
+        flags |= VK_ACCESS_SHADER_READ_BIT | VK_ACCESS_SHADER_WRITE_BIT;
+    if (bind_flags & WINED3D_BIND_INDIRECT_BUFFER)
+        flags |= VK_ACCESS_INDIRECT_COMMAND_READ_BIT;
+    if (bind_flags & WINED3D_BIND_RENDER_TARGET)
+        flags |= VK_ACCESS_COLOR_ATTACHMENT_READ_BIT | VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT;
+    if (bind_flags & WINED3D_BIND_DEPTH_STENCIL)
+        flags |= VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_READ_BIT | VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT;
+    if (bind_flags & WINED3D_BIND_STREAM_OUTPUT)
+        FIXME("Ignoring some bind flags %#x.\n", bind_flags);
+
+    return flags;
+}
