@@ -87,7 +87,6 @@
 #include "shlobj.h"
 #include "shellapi.h"
 #include "shlwapi.h"
-#include "wine/library.h"
 #include "wine/list.h"
 #include "wine/debug.h"
 #include "wine/unicode.h"
@@ -1849,13 +1848,12 @@ static BOOL request_selection_contents( Display *display, BOOL changed )
                last_size != size ||
                memcmp( last_data, data, size ));
 
-    if (!changed)
+    if (!changed || !OpenClipboard( clipboard_hwnd ))
     {
         HeapFree( GetProcessHeap(), 0, data );
         return FALSE;
     }
 
-    if (!OpenClipboard( clipboard_hwnd )) return FALSE;
     TRACE( "selection changed, importing\n" );
     EmptyClipboard();
     is_clipboard_owner = TRUE;
@@ -1987,14 +1985,14 @@ static void xfixes_init(void)
     int major = 3, minor = 0;
     void *handle;
 
-    handle = wine_dlopen(SONAME_LIBXFIXES, RTLD_NOW, NULL, 0);
+    handle = dlopen(SONAME_LIBXFIXES, RTLD_NOW);
     if (!handle) return;
 
-    pXFixesQueryExtension = wine_dlsym(handle, "XFixesQueryExtension", NULL, 0);
+    pXFixesQueryExtension = dlsym(handle, "XFixesQueryExtension");
     if (!pXFixesQueryExtension) return;
-    pXFixesQueryVersion = wine_dlsym(handle, "XFixesQueryVersion", NULL, 0);
+    pXFixesQueryVersion = dlsym(handle, "XFixesQueryVersion");
     if (!pXFixesQueryVersion) return;
-    pXFixesSelectSelectionInput = wine_dlsym(handle, "XFixesSelectSelectionInput", NULL, 0);
+    pXFixesSelectSelectionInput = dlsym(handle, "XFixesSelectSelectionInput");
     if (!pXFixesSelectSelectionInput) return;
 
     if (!pXFixesQueryExtension(clipboard_display, &event_base, &error_base))
